'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var test = _interopDefault(require('tape'));
var React = _interopDefault(require('react'));
var PropTypes = _interopDefault(require('prop-types'));
var ReactTestUtils = _interopDefault(require('react-dom/test-utils'));
var Styletron = _interopDefault(require('styletron-server'));
var styletronUtils = require('styletron-utils');

const STYLETRON_KEY = '__STYLETRON';

/**
 * Helper function to create styled element components
 * @packagename styletron-react
 * @param  {String|function} base        Tag name or component
 * @param  {function|object} styleFn     Style object or function that returns a style object
 * @param  {function}        assignProps Function that consumes the style result and props and returns an object with new props
 * @return {function}                    Component
 * @example
 */
function core(base, style, assignProps) {
  if (typeof base === 'function' && base[STYLETRON_KEY]) {
    var _base$STYLETRON_KEY = base[STYLETRON_KEY];
    const tag = _base$STYLETRON_KEY.tag,
          styles = _base$STYLETRON_KEY.styles;
    // Styled component

    return createStyledElementComponent(tag, styles.concat(style), assignProps);
  }
  if (typeof base === 'string' || typeof base === 'function') {
    // Tag name or non-styled component
    return createStyledElementComponent(base, [style], assignProps);
  }
  throw new Error('`styled` takes either a DOM element name or a component');
}

function createStyledElementComponent(tagName, stylesArray, assignProps) {
  function StyledElement(props, context) {
    const ownProps = assign({}, props);
    delete ownProps.innerRef;

    const styleResult = {};
    StyledElement[STYLETRON_KEY].styles.forEach(style => {
      if (typeof style === 'function') {
        assign(styleResult, style(ownProps, context));
      } else if (typeof style === 'object') {
        assign(styleResult, style);
      }
    });

    const elementProps = assignProps(context.styletron, styleResult, ownProps);

    if (props.innerRef) {
      elementProps.ref = props.innerRef;
    }

    if (typeof StyledElement[STYLETRON_KEY].tag === 'string') {
      // Use custom element workaround to avoid warnings for "invalid" attributes
      elementProps.is = true;
      // Because of this, we need to use true `class` and `for` attributes
      elementProps.class = elementProps.className;
      elementProps.className = void 0;
      elementProps.for = elementProps.htmlFor;
    }

    return React.createElement(StyledElement[STYLETRON_KEY].tag, elementProps);
  }

  StyledElement[STYLETRON_KEY] = {
    tag: tagName,
    styles: stylesArray
  };

  StyledElement.contextTypes = { styletron: PropTypes.object };

  return StyledElement;
}

function assign(target, source) {
  for (const key in source) {
    target[key] = source[key];
  }
  return target;
}

/**
 * Helper function to create styled element components
 * @packagename styletron-react
 * @param  {String|function} base     Tag name or styled element component
 * @param  {function|object} styleFn  Style object or function that returns a style object
 * @return {function}                 Styled element component
 * @example
 * import {styled} from 'styletron-react';
 *
 * const Panel = styled('div', {
 *   backgroundColor: 'lightblue',
 *   fontSize: '12px'
 * });
 *
 * <Panel>Hello World</Panel>
 * @example
 * import {styled} from 'styletron-react';
 *
 * const Panel = styled('div', (props) => ({
 *   backgroundColor: props.alert ? 'orange' : 'lightblue',
 *   fontSize: '12px'
 * }));
 *
 * <Panel alert>Danger!</Panel>
 * @example
 * import {styled} from 'styletron-react';
 *
 * const DeluxePanel = styled(Panel, (props) => ({
 *   backgroundColor: props.alert ? 'firebrick' : 'rebeccapurple',
 *   color: 'white',
 *   boxShadow: '3px 3px 3px darkgray'
 * }));
 *
 * <DeluxePanel>Bonjour Monde</DeluxePanel>
 */
function styled(base, style) {
  return core(base, style, assignProps);
}

function assignProps(styletron, styleResult, ownProps) {
  const styletronClassName = styletronUtils.injectStylePrefixed(styletron, styleResult);
  // Skipping cloning of `ownProps` since that's already done internally
  ownProps.className = ownProps.className ? `${ownProps.className} ${styletronClassName}` : styletronClassName;
  return ownProps;
}

/**
 * @class StyletronProvider
 * @packagename styletron-react
 * @description Provides a Styletron instance to descendant styled components via context
 * @example
 * const Styletron = require('styletron');
 *
 * function render() {
 *   return React.renderToString(
 *     <StyletronProvider styletron={new Styletron()}>
 *       <App/>
 *     </StyletronProvider>
 *   );
 * }
 * 
 * @property {object} styletron - Styletron instance
 * @property {ReactElement} children - children
 * @extends ReactClass
 */
class StyletronProvider extends React.Component {
  getChildContext() {
    return { styletron: this.styletron };
  }
  constructor(props, context) {
    super(props, context);
    this.styletron = props.styletron;
  }
  render() {
    return React.Children.only(this.props.children);
  }
}

StyletronProvider.PropTypes = {
  styletron: PropTypes.object.isRequired,
  children: PropTypes.element.isRequired
};

StyletronProvider.childContextTypes = {
  styletron: PropTypes.object.isRequired
};

/* eslint-env browser */

function strictAssignProps(styletron, styleResult, ownProps) {
  return Object.assign({}, ownProps, {
    className: styletronUtils.injectStylePrefixed(styletron, styleResult)
  });
}

test('provider provides instance', t => {
  const mockInstance = {};
  const MockComponent = (props, context) => {
    t.equal(context.styletron, mockInstance, 'styletron instance override provided');
    return React.createElement('div');
  };
  MockComponent.contextTypes = { styletron: PropTypes.object };
  ReactTestUtils.renderIntoDocument(React.createElement(StyletronProvider, {
    styletron: mockInstance
  }, React.createElement(MockComponent)));
  t.end();
});

test('props passed to core function', t => {
  t.plan(1);
  const expected = {
    prop1: 'foo'
  };
  const Widget = core('div', props => {
    t.deepEqual(props, expected, 'props accessible in core fn');
    return {};
  }, strictAssignProps);
  const styletron = new Styletron();
  ReactTestUtils.renderIntoDocument(React.createElement(StyletronProvider, { styletron }, React.createElement(Widget, expected)));
});

test('core applies styles', t => {
  const Widget = core('div', () => {
    return { color: 'red' };
  }, strictAssignProps);
  const styletron = new Styletron();
  const output = ReactTestUtils.renderIntoDocument(React.createElement(StyletronProvider, { styletron }, React.createElement(Widget)));
  const div = ReactTestUtils.findRenderedDOMComponentWithTag(output, 'div');
  t.equal(div.className, 'a', 'styletron classes');
  t.equal(styletron.getCss(), '.a{color:red}');
  t.end();
});

test('core applies static styles', t => {
  const Widget = core('div', { color: 'red' }, strictAssignProps);
  const styletron = new Styletron();
  const output = ReactTestUtils.renderIntoDocument(React.createElement(StyletronProvider, { styletron }, React.createElement(Widget)));
  const div = ReactTestUtils.findRenderedDOMComponentWithTag(output, 'div');
  t.equal(div.className, 'a', 'matches expected styletron classes');
  t.equal(styletron.getCss(), '.a{color:red}');
  t.end();
});

test('core passes through valid props', t => {
  const Widget = core('div', { color: 'red' }, strictAssignProps);
  const styletron = new Styletron();
  const output = ReactTestUtils.renderIntoDocument(React.createElement(StyletronProvider, { styletron }, React.createElement(Widget, {
    'data-bar': 'bar'
  })));
  const div = ReactTestUtils.findRenderedDOMComponentWithTag(output, 'div');
  t.equal(div.getAttribute('data-bar'), 'bar', 'valid attribute prop passed through');
  t.end();
});

test('core composition', t => {
  const Widget = core('div', { color: 'red', display: 'inline' }, strictAssignProps);
  const SuperWidget = core(Widget, { display: 'block', background: 'black' }, strictAssignProps);
  const styletron = new Styletron();
  const output = ReactTestUtils.renderIntoDocument(React.createElement(StyletronProvider, { styletron }, React.createElement(SuperWidget)));
  const div = ReactTestUtils.findRenderedDOMComponentWithTag(output, 'div');
  t.equal(div.className, 'a b c', 'matches expected styletron classes');
  t.equal(styletron.getCss(), '.a{color:red}.b{display:block}.c{background:black}');
  t.end();
});

test('core component', t => {
  const Widget = ({ className }) => React.createElement('div', { className });
  const SuperWidget = core(Widget, { color: 'red' }, strictAssignProps);
  const styletron = new Styletron();
  const output = ReactTestUtils.renderIntoDocument(React.createElement(StyletronProvider, { styletron }, React.createElement(SuperWidget)));
  const div = ReactTestUtils.findRenderedDOMComponentWithTag(output, 'div');
  t.equal(div.className, 'a', 'matches expected styletron classes');
  t.equal(styletron.getCss(), '.a{color:red}');
  t.end();
});

test('innerRef works', t => {
  t.plan(1);

  const Widget = core('button', { color: 'red' }, strictAssignProps);
  const styletron = new Styletron();

  class TestComponent extends React.Component {
    componentDidMount() {
      t.ok(this.widgetInner instanceof HTMLButtonElement, 'is button');
    }

    render() {
      return React.createElement(Widget, {
        innerRef: c => {
          this.widgetInner = c;
        }
      });
    }
  }

  ReactTestUtils.renderIntoDocument(React.createElement(StyletronProvider, { styletron }, React.createElement(TestComponent)));
});

test('innerRef not passed', t => {
  t.plan(2);

  class InnerComponent extends React.Component {
    render() {
      t.deepEqual(this.props, {
        className: 'a',
        foo: 'bar'
      }, 'props match expected');
      return React.createElement(
        'button',
        null,
        'InnerComponent'
      );
    }
  }

  const Widget = core(InnerComponent, { color: 'red' }, strictAssignProps);
  const styletron = new Styletron();

  class TestComponent extends React.Component {
    componentDidMount() {
      t.ok(ReactTestUtils.isCompositeComponentWithType(this.widgetInner, InnerComponent), 'is InnerComponent');
    }

    render() {
      return React.createElement(Widget, {
        foo: 'bar',
        innerRef: c => {
          this.widgetInner = c;
        }
      });
    }
  }

  ReactTestUtils.renderIntoDocument(React.createElement(StyletronProvider, { styletron }, React.createElement(TestComponent)));
});

test('styled merges class name prop', t => {
  const Widget = styled('div', { color: 'red' });
  const styletron = new Styletron();
  const output = ReactTestUtils.renderIntoDocument(React.createElement(StyletronProvider, { styletron }, React.createElement(Widget, { className: 'foo' })));
  const div = ReactTestUtils.findRenderedDOMComponentWithTag(output, 'div');
  t.equal(div.className, 'foo a', 'matches expected classes');
  t.end();
});
//# sourceMappingURL=node.js.map
